@@ -32,6 +32,11 @@
 #include "common/strings/editing.h"
 #include "common/strings/parsing.h"
 
+union double_to_uint64_t {
+  uint64_t i;
+  double d;
+};
+
 #if !defined(SYS_WINDOWS)
 static std::string
 get_errno_msg() {
@@ -147,7 +152,7 @@ mm_file_io_c::setup() {
 void
 mm_file_io_c::prepare_path(const std::string &path) {
   boost::filesystem::path directory = boost::filesystem::path(path).parent_path();
-  if (boost::filesystem::exists(directory))
+  if (directory.empty() || boost::filesystem::exists(directory))
     return;
 
   boost::system::error_code error_code;
@@ -156,6 +161,16 @@ mm_file_io_c::prepare_path(const std::string &path) {
     throw mtx::mm_io::create_directory_x(path, strerror(error_code.value()), error_code.value());
 }
 
+memory_cptr
+mm_file_io_c::slurp(std::string const &file_name) {
+  mm_file_io_c in(file_name, MODE_READ);
+  auto content = memory_c::alloc(in.get_size());
+  if (in.get_size() != in.read(content, in.get_size()))
+    throw mtx::mm_io::end_of_file_x{};
+
+  return content;
+}
+
 uint64
 mm_file_io_c::getFilePointer() {
   return m_current_position;
@@ -365,6 +380,30 @@ mm_io_c::read_uint64_be() {
   return get_uint64_be(buffer);
 }
 
+double
+mm_io_c::read_double() {
+  double_to_uint64_t d2ui;
+
+  d2ui.i = read_uint64_be();
+  return d2ui.d;
+}
+
+unsigned int
+mm_io_c::read_mp4_descriptor_len() {
+  unsigned int len       = 0;
+  unsigned int num_bytes = 0;
+  uint8_t byte;
+
+  do {
+    byte = read_uint8();
+    len  = (len << 7) | (byte & 0x7f);
+    ++num_bytes;
+
+  } while (((byte & 0x80) == 0x80) && (4 > num_bytes));
+
+  return len;
+}
+
 uint32_t
 mm_io_c::read(memory_cptr &buffer,
               size_t size,
@@ -436,6 +475,13 @@ mm_io_c::write_uint64_be(uint64_t value) {
   return write(&buffer, sizeof(uint64_t));
 }
 
+int
+mm_io_c::write_double(double value) {
+  double_to_uint64_t d2ui;
+  d2ui.d = value;
+  return write_uint64_be(d2ui.i);
+}
+
 size_t
 mm_io_c::write(const void *buffer,
                size_t size) {
@@ -662,6 +708,20 @@ mm_mem_io_c::mm_mem_io_c(const unsigned char *mem,
     throw mtx::invalid_parameter_x();
 }
 
+mm_mem_io_c::mm_mem_io_c(memory_c const &mem)
+  : m_pos{}
+  , m_mem_size{mem.get_size()}
+  , m_allocated{mem.get_size()}
+  , m_increase{}
+  , m_mem{}
+  , m_ro_mem{mem.get_buffer()}
+  , m_free_mem{}
+  , m_read_only{true}
+{
+  if (!m_ro_mem)
+    throw mtx::invalid_parameter_x{};
+}
+
 mm_mem_io_c::~mm_mem_io_c() {
   close();
 }
