@@ -149,6 +149,8 @@ public:
   virtual uint32_t read_uint24_be();
   virtual uint32_t read_uint32_be();
   virtual uint64_t read_uint64_be();
+  virtual double read_double();
+  virtual unsigned int read_mp4_descriptor_len();
   virtual int write_uint8(unsigned char value);
   virtual int write_uint16_le(uint16_t value);
   virtual int write_uint32_le(uint32_t value);
@@ -156,6 +158,7 @@ public:
   virtual int write_uint16_be(uint16_t value);
   virtual int write_uint32_be(uint32_t value);
   virtual int write_uint64_be(uint64_t value);
+  virtual int write_double(double value);
   virtual void skip(int64 numbytes);
   virtual size_t write(const void *buffer, size_t size);
   virtual size_t write(std::string const &buffer) {
@@ -213,6 +216,7 @@ public:
   virtual ~mm_file_io_c();
 
   static void prepare_path(const std::string &path);
+  static memory_cptr slurp(std::string const &file_name);
 
   virtual uint64 getFilePointer();
 #if defined(SYS_WINDOWS)
@@ -310,6 +314,7 @@ protected:
 public:
   mm_mem_io_c(unsigned char *mem, uint64_t mem_size, int increase);
   mm_mem_io_c(const unsigned char *mem, uint64_t mem_size);
+  mm_mem_io_c(memory_c const &mem);
   ~mm_mem_io_c();
 
   virtual uint64 getFilePointer();
