@@ -33,8 +33,8 @@ xtr_flac_c::xtr_flac_c(const std::string &_codec_id,
 void
 xtr_flac_c::create_file(xtr_base_c *_master,
                         KaxTrackEntry &track) {
-  KaxCodecPrivate *priv = FINDFIRST(&track, KaxCodecPrivate);
-  if (nullptr == priv)
+  KaxCodecPrivate *priv = FindChild<KaxCodecPrivate>(&track);
+  if (!priv)
     mxerror(boost::format(Y("Track %1% with the CodecID '%2%' is missing the \"codec private\" element and cannot be extracted.\n")) % m_tid % m_codec_id);
 
   xtr_base_c::create_file(_master, track);
@@ -68,8 +68,8 @@ xtr_oggbase_c::create_file(xtr_base_c *master,
 void
 xtr_oggbase_c::create_standard_file(xtr_base_c *master,
                                     KaxTrackEntry &track) {
-  KaxCodecPrivate *priv = FINDFIRST(&track, KaxCodecPrivate);
-  if (nullptr == priv)
+  KaxCodecPrivate *priv = FindChild<KaxCodecPrivate>(&track);
+  if (!priv)
     mxerror(boost::format(Y("Track %1% with the CodecID '%2%' is missing the \"codec private\" element and cannot be extracted.\n")) % m_tid % m_codec_id);
 
   init_content_decoder(track);
@@ -200,7 +200,7 @@ xtr_oggbase_c::write_queued_frame(bool eos) {
   write_pages();
 
   m_queued_granulepos = -1;
-  m_queued_frame.clear();
+  m_queued_frame.reset();
 }
 
 // ------------------------------------------------------------------------
