@@ -40,6 +40,7 @@
 #include <matroska/KaxTrackAudio.h>
 #include <matroska/KaxTrackVideo.h>
 
+#include "common/alac.h"
 #include "common/at_scope_exit.h"
 #include "common/chapters/chapters.h"
 #include "common/ebml.h"
@@ -57,6 +58,7 @@
 #include "merge/pr_generic.h"
 #include "output/p_aac.h"
 #include "output/p_ac3.h"
+#include "output/p_alac.h"
 #include "output/p_avc.h"
 #include "output/p_dirac.h"
 #include "output/p_dts.h"
@@ -324,6 +326,19 @@ kax_reader_c::verify_acm_audio_track(kax_track_t *t) {
   return true;
 }
 
+bool
+kax_reader_c::verify_alac_audio_track(kax_track_t *t) {
+  t->a_formattag = FOURCC('A', 'L', 'A', 'C');
+
+  if (t->private_data && (sizeof(alac::codec_config_t) <= t->private_size))
+    return true;
+
+  if (verbose)
+    mxwarn(boost::format(Y("matroska_reader: The CodecID for track %1% is '%2%', but there was no codec private headers.\n")) % t->tnum % MKV_A_ALAC);
+
+  return false;
+}
+
 bool
 kax_reader_c::verify_flac_audio_track(kax_track_t *t) {
 #if defined(HAVE_FLAC_FORMAT_H)
@@ -378,6 +393,8 @@ kax_reader_c::verify_audio_track(kax_track_t *t) {
     t->a_formattag = 0x0055;
   else if (balg::starts_with(t->codec_id, MKV_A_AC3) || (t->codec_id == MKV_A_EAC3))
     t->a_formattag = 0x2000;
+  else if (t->codec_id == MKV_A_ALAC)
+    is_ok = verify_alac_audio_track(t);
   else if (t->codec_id == MKV_A_DTS)
     t->a_formattag = 0x2001;
   else if (t->codec_id == MKV_A_PCM)
@@ -1555,6 +1572,13 @@ kax_reader_c::create_ac3_audio_packetizer(kax_track_t *t,
   show_packetizer_info(t->tnum, t->ptzr_ptr);
 }
 
+void
+kax_reader_c::create_alac_audio_packetizer(kax_track_t *t,
+                                          track_info_c &nti) {
+  set_track_packetizer(t, new alac_packetizer_c(this, nti, memory_c::clone(t->private_data, t->private_size), t->a_sfreq, t->a_channels));
+  show_packetizer_info(t->tnum, t->ptzr_ptr);
+}
+
 void
 kax_reader_c::create_dts_audio_packetizer(kax_track_t *t,
                                           track_info_c &nti) {
@@ -1607,7 +1631,7 @@ kax_reader_c::create_mp3_audio_packetizer(kax_track_t *t,
 void
 kax_reader_c::create_pcm_audio_packetizer(kax_track_t *t,
                                           track_info_c &nti) {
-  set_track_packetizer(t, new pcm_packetizer_c(this, nti, t->a_sfreq, t->a_channels, t->a_bps, false, 0x0003 == t->a_formattag));
+  set_track_packetizer(t, new pcm_packetizer_c(this, nti, t->a_sfreq, t->a_channels, t->a_bps, 0x0003 == t->a_formattag));
   show_packetizer_info(t->tnum, t->ptzr_ptr);
 }
 
@@ -1670,6 +1694,9 @@ kax_reader_c::create_audio_packetizer(kax_track_t *t,
   else if (0xFFFE == t->a_formattag)
     create_vorbis_audio_packetizer(t, nti);
 
+  else if (FOURCC('A', 'L', 'A', 'C') == t->a_formattag)
+    create_alac_audio_packetizer(t, nti);
+
   else if ((FOURCC('M', 'P', '4', 'A') == t->a_formattag) || (0x00ff == t->a_formattag))
     create_aac_audio_packetizer(t, nti);
 
