@@ -50,7 +50,6 @@ cluster_helper_c::cluster_helper_c()
   , m_num_cue_elements(0)
   , m_header_overhead(-1)
   , m_timecode_offset(0)
-  , m_previous_packets(nullptr)
   , m_bytes_in_file(0)
   , m_first_timecode_in_file(-1)
   , m_first_discarded_timecode{-1}
@@ -58,11 +57,11 @@ cluster_helper_c::cluster_helper_c()
   , m_discarded_duration{0}
   , m_min_timecode_in_cluster(-1)
   , m_max_timecode_in_cluster(-1)
-  , m_attachments_size(0)
   , m_first_video_keyframe_seen{}
   , m_out(nullptr)
   , m_current_split_point(m_split_points.begin())
   , m_discarding{false}
+  , m_splitting_and_processed_fully{false}
   , m_debug_splitting{debugging_requested("cluster_helper|splitting")}
   , m_debug_packets{  debugging_requested("cluster_helper|cluster_helper_packets")}
   , m_debug_duration{ debugging_requested("cluster_helper|cluster_helper_duration")}
@@ -185,6 +184,13 @@ cluster_helper_c::split(packet_cptr &packet) {
     finish_file();
 
   if (m_current_split_point->m_use_once) {
+    if (   m_current_split_point->m_discard
+        && (split_point_t::SPT_PARTS == m_current_split_point->m_type)
+        && (m_split_points.end()     == (m_current_split_point + 1))) {
+      mxdebug_if(m_debug_splitting, boost::format("Splitting: Last part in 'parts:' splitting mode finished\n"));
+      m_splitting_and_processed_fully = true;
+    }
+
     m_discarding = m_current_split_point->m_discard;
     ++m_current_split_point;
   }
@@ -577,6 +583,11 @@ cluster_helper_c::split_mode_produces_many_files()
   return false;
 }
 
+void
+cluster_helper_c::discard_queued_packets() {
+  m_packets.clear();
+}
+
 void
 cluster_helper_c::dump_split_points()
   const {
