@@ -83,17 +83,17 @@ private:
   int m_cluster_content_size;
   int64_t m_max_timecode_and_duration, m_max_video_timecode_rendered;
   int64_t m_previous_cluster_tc, m_num_cue_elements, m_header_overhead;
-  int64_t m_timecode_offset, *m_previous_packets;
+  int64_t m_timecode_offset;
   int64_t m_bytes_in_file, m_first_timecode_in_file, m_first_discarded_timecode, m_last_discarded_timecode_and_duration, m_discarded_duration;
   int64_t m_min_timecode_in_cluster, m_max_timecode_in_cluster;
-  int64_t m_attachments_size;
   bool m_first_video_keyframe_seen;
   mm_io_c *m_out;
 
   std::vector<split_point_t> m_split_points;
   std::vector<split_point_t>::iterator m_current_split_point;
 
-  bool m_discarding, m_debug_splitting, m_debug_packets, m_debug_duration, m_debug_rendering;
+  bool m_discarding, m_splitting_and_processed_fully;
+  bool m_debug_splitting, m_debug_packets, m_debug_duration, m_debug_rendering;
 
 public:
   cluster_helper_c();
@@ -133,6 +133,11 @@ public:
     return m_packets.size();
   }
 
+  void discard_queued_packets();
+  bool is_splitting_and_processed_fully() const {
+    return m_splitting_and_processed_fully;
+  }
+
 private:
   void set_duration(render_groups_c *rg);
   bool must_duration_be_set(render_groups_c *rg, packet_cptr &new_packet);
