@@ -433,10 +433,10 @@ identify(std::string &filename) {
 /** \brief Parse a number postfixed with a time-based unit
 
    This function parsers a number that is postfixed with one of the
-   four units 's', 'ms', 'us' or 'ns'. If the postfix is 'fps' then
-   this means 'frames per second'. If the postfix is 'p' or 'i' then
-   it is also interpreted as the number of 'progressive frames per
-   second' or 'interlaced frames per second'.
+   units 's', 'ms', 'us', 'ns', 'fps', 'p' or 'i'. If the postfix is
+   'fps' then this means 'frames per second'. If the postfix is 'p' or
+   'i' then it is also interpreted as the number of 'progressive
+   frames per second' or 'interlaced frames per second'.
 
    It returns a number of nanoseconds.
 */
@@ -497,7 +497,7 @@ parse_number_with_unit(const std::string &s,
       return (int64_t)(1000000000.0 / d_value);
 
   } else if (unit != "s")
-    mxerror(boost::format(Y("'%1%' does not contain a valid unit ('s', 'ms', 'us' or 'ns') in '%2% %3%'.\n")) % s % argument % display_s);
+    mxerror(boost::format(Y("'%1%' does not contain a valid unit ('s', 'ms', 'us', 'ns', 'fps', 'p' or 'i') in '%2% %3%'.\n")) % s % argument % display_s);
 
   if (is_fraction)
     return multiplier * n / d;
@@ -682,7 +682,7 @@ parse_arg_display_dimensions(const std::string s,
 
   std::vector<std::string> dims = split(parts[1], "x", 2);
   int64_t id = 0;
-  int w, h;
+  int w = 0, h = 0;
   if ((dims.size() != 2) || !parse_number(parts[0], id) || !parse_number(dims[0], w) || !parse_number(dims[1], h) || (0 >= w) || (0 >= h))
     mxerror(boost::format(Y("Display dimensions: not given in the form <TID>:<width>x<height>, e.g. 1:640x480 (argument was '%1%').\n")) % s);
 
@@ -2288,7 +2288,7 @@ parse_args(std::vector<std::string> args) {
 static std::vector<std::string>
 setup(int argc,
       char **argv) {
-  clear_list_of_unique_uint32(UNIQUE_ALL_IDS);
+  clear_list_of_unique_numbers(UNIQUE_ALL_IDS);
 
   mtx_common_init("mkvmerge");
   g_kax_tracks = new KaxTracks();
