@@ -77,6 +77,7 @@
 #include "input/r_avc.h"
 #include "input/r_avi.h"
 #include "input/r_cdxa.h"
+#include "input/r_coreaudio.h"
 #include "input/r_dirac.h"
 #include "input/r_dts.h"
 #include "input/r_dv.h"
@@ -186,6 +187,7 @@ int g_file_num = 1;
 int g_split_max_num_files                   = 65535;
 
 append_mode_e g_append_mode                 = APPEND_MODE_FILE_BASED;
+bool s_appending_files                      = false;
 
 bool g_stereo_mode_used                     = false;
 
@@ -377,6 +379,8 @@ get_file_type(filelist_t &file) {
     type = FILE_TYPE_WAVPACK4;
   else if (ivf_reader_c::probe_file(io, size))
     type = FILE_TYPE_IVF;
+  else if (coreaudio_reader_c::probe_file(io, size))
+    type = FILE_TYPE_COREAUDIO;
   else if (dirac_es_reader_c::probe_file(io, size))
     type = FILE_TYPE_DIRAC;
   // File types that are misdetected sometimes and that aren't supported
@@ -565,11 +569,11 @@ add_attachment(attachment_t attachment) {
            && (ex_attachment.data->get_size() == attachment.data->get_size())))
         return attachment.id;
 
-    add_unique_uint32(attachment.id, UNIQUE_ATTACHMENT_IDS);
+    add_unique_number(attachment.id, UNIQUE_ATTACHMENT_IDS);
 
   } else
     // No ID yet. Let's assign one.
-    attachment.id = create_unique_uint32(UNIQUE_ATTACHMENT_IDS);
+    attachment.id = create_unique_number(UNIQUE_ATTACHMENT_IDS);
 
   g_attachments.push_back(attachment);
 
@@ -631,7 +635,7 @@ set_timecode_scale() {
       && (0 < highest_sample_rate)
       && (   !video_present
           || (TIMECODE_SCALE_MODE_AUTO == g_timecode_scale_mode)))
-    g_timecode_scale = static_cast<int64_t>(1000000000.0d / highest_sample_rate - 1.0);
+    g_timecode_scale = static_cast<int64_t>(1000000000.0 / highest_sample_rate - 1.0);
 
   g_max_ns_per_cluster                                    = std::min((int64_t)(32700 * g_timecode_scale), g_max_ns_per_cluster);
   GetChildAs<KaxTimecodeScale, EbmlUInteger>(s_kax_infos) = (int64_t)g_timecode_scale;
@@ -1178,6 +1182,9 @@ create_readers() {
         case FILE_TYPE_AVI:
           file.reader = new avi_reader_c(*file.ti, input_file);
           break;
+        case FILE_TYPE_COREAUDIO:
+          file.reader = new coreaudio_reader_c(*file.ti, input_file);
+          break;
         case FILE_TYPE_DIRAC:
           file.reader = new dirac_es_reader_c(*file.ti, input_file);
           break;
@@ -1278,6 +1285,9 @@ create_readers() {
     for (auto &file : g_files) {
       file.reader->m_appending = file.appending;
       file.reader->create_packetizers();
+
+      if (!s_appending_files)
+        s_appending_files = file.appending;
     }
     // Check if all track IDs given on the command line are actually
     // present.
@@ -1935,6 +1945,74 @@ establish_deferred_connections(filelist_t &file) {
   // \todo Select a new file that the subs will defer to.
 }
 
+static void
+pull_packetizers_for_packets() {
+  for (auto &ptzr : g_packetizers) {
+    if (FILE_STATUS_HOLDING == ptzr.status)
+      ptzr.status = FILE_STATUS_MOREDATA;
+
+    ptzr.old_status = ptzr.status;
+
+    while (   !ptzr.pack
+           && (FILE_STATUS_MOREDATA == ptzr.status)
+           && !ptzr.packetizer->packet_available())
+      ptzr.status = ptzr.packetizer->read();
+
+    if (   (FILE_STATUS_MOREDATA != ptzr.status)
+           && (FILE_STATUS_MOREDATA == ptzr.old_status))
+      ptzr.packetizer->force_duration_on_last_packet();
+
+    if (!ptzr.pack)
+      ptzr.pack = ptzr.packetizer->get_packet();
+
+    if (!ptzr.pack && (FILE_STATUS_DONE == ptzr.status))
+      ptzr.status = FILE_STATUS_DONE_AND_DRY;
+
+    // Has this packetizer changed its status from "data available" to
+    // "file done" during this loop? If so then decrease the number of
+    // unfinished packetizers in the corresponding file structure.
+    if (   (FILE_STATUS_DONE_AND_DRY == ptzr.status)
+        && (ptzr.old_status != ptzr.status)) {
+      filelist_t &file = g_files[ptzr.file];
+      file.num_unfinished_packetizers--;
+
+      // If all packetizers for a file have finished then establish the
+      // deferred connections.
+      if ((0 >= file.num_unfinished_packetizers) && (0 < file.old_num_unfinished_packetizers)) {
+        establish_deferred_connections(file);
+        file.done = true;
+      }
+      file.old_num_unfinished_packetizers = file.num_unfinished_packetizers;
+    }
+  }
+}
+
+static packetizer_t *
+select_winning_packetizer() {
+  packetizer_t *winner = nullptr;
+
+  for (auto &ptzr : g_packetizers) {
+    if (!ptzr.pack)
+      continue;
+
+    if (!winner || !winner->pack)
+      winner = &ptzr;
+
+    else if (ptzr.pack && (ptzr.pack->assigned_timecode < winner->pack->assigned_timecode))
+      winner = &ptzr;
+  }
+
+  return winner;
+}
+
+static void
+discard_queued_packets() {
+  for (auto &ptzr : g_packetizers)
+    ptzr.packetizer->discard_queued_packets();
+
+  g_cluster_helper->discard_queued_packets();
+}
+
 /** \brief Request packets and handle the next one
 
    Requests packets from each packetizer, selects the packet with the
@@ -1949,60 +2027,14 @@ main_loop() {
 
     // Step 1: Make sure a packet is available for each output
     // as long we haven't already processed the last one.
-    for (auto &ptzr : g_packetizers) {
-      if (FILE_STATUS_HOLDING == ptzr.status)
-        ptzr.status = FILE_STATUS_MOREDATA;
-
-      ptzr.old_status = ptzr.status;
-
-      while (   !ptzr.pack
-             && (FILE_STATUS_MOREDATA == ptzr.status)
-             && !ptzr.packetizer->packet_available())
-        ptzr.status = ptzr.packetizer->read();
-
-      if (   (FILE_STATUS_MOREDATA != ptzr.status)
-          && (FILE_STATUS_MOREDATA == ptzr.old_status))
-        ptzr.packetizer->force_duration_on_last_packet();
-
-      if (!ptzr.pack)
-        ptzr.pack = ptzr.packetizer->get_packet();
-
-      if (!ptzr.pack && (FILE_STATUS_DONE == ptzr.status))
-        ptzr.status = FILE_STATUS_DONE_AND_DRY;
-
-      // Has this packetizer changed its status from "data available" to
-      // "file done" during this loop? If so then decrease the number of
-      // unfinished packetizers in the corresponding file structure.
-      if (   (FILE_STATUS_DONE_AND_DRY == ptzr.status)
-          && (ptzr.old_status != ptzr.status)) {
-        filelist_t &file = g_files[ptzr.file];
-        file.num_unfinished_packetizers--;
-
-        // If all packetizers for a file have finished then establish the
-        // deferred connections.
-        if ((0 >= file.num_unfinished_packetizers) && (0 < file.old_num_unfinished_packetizers)) {
-          establish_deferred_connections(file);
-          file.done = true;
-        }
-        file.old_num_unfinished_packetizers = file.num_unfinished_packetizers;
-      }
-    }
+    pull_packetizers_for_packets();
 
     // Step 2: Pick the packet with the lowest timecode and
     // stuff it into the Matroska file.
-    packetizer_t * winner = nullptr;
-    for (auto &ptzr : g_packetizers) {
-      if (ptzr.pack) {
-        if (!winner || !winner->pack)
-          winner = &ptzr;
-
-        else if (ptzr.pack && (ptzr.pack->assigned_timecode < winner->pack->assigned_timecode))
-          winner = &ptzr;
-      }
-    }
+    auto winner = select_winning_packetizer();
 
     // Append the next track if appending is wanted.
-    bool appended_a_track = append_tracks_maybe();
+    bool appended_a_track = s_appending_files && append_tracks_maybe();
 
     if (winner && winner->pack) {
       packet_cptr pack = winner->pack;
@@ -2013,6 +2045,13 @@ main_loop() {
 
       winner->pack.reset();
 
+      // If splitting by parts is active and the last part has been
+      // processed fully then we can finish up.
+      if (g_cluster_helper->is_splitting_and_processed_fully()) {
+        discard_queued_packets();
+        break;
+      }
+
       // display some progress information
       if (1 <= verbose)
         display_progress();
