@@ -11,7 +11,7 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "common/os.h"
+#include "common/common_pch.h"
 
 #include <wx/wx.h>
 #include <wx/config.h>
@@ -22,7 +22,6 @@
 # include <ApplicationServices/ApplicationServices.h>
 #endif
 
-#include "common/common_pch.h"
 #include "common/chapters/chapters.h"
 #include "common/command_line.h"
 #include "common/common_pch.h"
@@ -33,7 +32,6 @@
 #include "common/strings/formatting.h"
 #include "common/translation.h"
 #include "common/wx.h"
-#include "common/xml/element_mapping.h"
 #include "mmg/header_editor/frame.h"
 #include "mmg/mmg_dialog.h"
 #include "mmg/mmg.h"
@@ -109,7 +107,7 @@ mmg_app::init_ui_locale() {
 
   const wxLanguageInfo *lang_info = wxLocale::FindLanguageInfo(wxU(m_ui_locale));
   if (s_first_init) {
-    if ((nullptr != lang_info) && m_locale.Init(lang_info->Language)) {
+    if (lang_info && m_locale.Init(lang_info->Language)) {
       m_locale.AddCatalog(wxU("wxstd"));
 #ifdef SYS_WINDOWS
       m_locale.AddCatalog(wxU("wxmsw"));
@@ -164,7 +162,9 @@ mmg_app::OnInit() {
   TransformProcessType(&PSN, kProcessTransformToForegroundApplication);
 #endif
 
-  mtx_common_init();
+  wxImage::AddHandler(new wxPNGHandler);
+
+  mtx_common_init("mmg");
 
   wxConfigBase *cfg;
   uint32_t i;
