@@ -11,14 +11,7 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-// This order is required. wx/wxprec.h includes wx's winundef.h which
-// cannot be included after windows.h has been included. But
-// mkvtoolnix' common.h includes windows.h.
-#undef __STRICT_ANSI__
-
-#include <wx/wxprec.h>
-
-#include "common/common.h"
+#include "common/common_pch.h"
 
 #include <wx/wx.h>
 #include <wx/dnd.h>
@@ -82,9 +75,8 @@ tab_attachments::tab_attachments(wxWindow *parent):
   st_mimetype    = new wxStaticText(this, wxID_STATIC, wxEmptyString);
   cob_mimetype   = new wxMTX_COMBOBOX_TYPE(this, ID_CB_MIMETYPE, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, nullptr, wxCB_DROPDOWN);
   cob_mimetype->Append(wxEmptyString);
-  int i;
-  for (i = 0; mime_types[i].name != nullptr; i++)
-    cob_mimetype->Append(wxU(mime_types[i].name));
+  for (auto &mime_type : mime_types)
+    cob_mimetype->Append(wxU(mime_type.name));
   cob_mimetype->SetSizeHints(0, -1);
 
   st_style  = new wxStaticText(this, wxID_STATIC, wxEmptyString);
@@ -414,7 +406,7 @@ void
 tab_attachments::remove_attached_files_for(mmg_file_cptr &f) {
   int i;
   for (i = m_attached_files.size() - 1; 0 <= i; --i) {
-    if (m_attached_files[i]->source == f.get_object()) {
+    if (m_attached_files[i]->source == f.get()) {
       clb_attached_files->Delete(i);
       m_attached_files.erase(m_attached_files.begin() + i);
     }
