@@ -41,14 +41,15 @@ public:
   virtual void set_skipping_is_normal(bool skipping_is_normal) {
     m_skipping_is_normal = skipping_is_normal;
   }
-  virtual void flush();
-
   virtual const std::string get_format_name(bool translate = true) {
     return translate ? Y("DTS") : "DTS";
   }
 
   virtual connection_result_e can_connect_to(generic_packetizer_c *src, std::string &error_message);
 
+protected:
+  virtual void flush_impl();
+
 private:
   virtual unsigned char *get_dts_packet(dts_header_t &dts_header, bool flushing);
   virtual void process_available_packets(bool flushing);
