@@ -33,8 +33,6 @@ public:
   virtual int process(packet_cptr packet);
   virtual void set_headers();
 
-  virtual void flush();
-
   virtual const std::string get_format_name(bool translate = true) {
     return translate ? Y("VC1") : "VC1";
   };
@@ -42,6 +40,7 @@ public:
   virtual connection_result_e can_connect_to(generic_packetizer_c *src, std::string &error_message);
 
 protected:
+  virtual void flush_impl();
   virtual void flush_frames();
   virtual void headers_found();
   virtual void add_timecodes_to_parser(packet_cptr &packet);
