@@ -14,12 +14,14 @@
 #include "common/common_pch.h"
 
 #include "common/cli_parser.h"
+#include "propedit/attachment_target.h"
 #include "propedit/options.h"
 
 class propedit_cli_parser_c: public cli_parser_c {
 protected:
   options_cptr m_options;
   target_cptr m_target;
+  attachment_target_c::options_t m_attachment;
 
 public:
   propedit_cli_parser_c(const std::vector<std::string> &args);
@@ -28,6 +30,7 @@ public:
 
 protected:
   void init_parser();
+  void validate();
 
   void add_target();
   void add_change();
@@ -36,6 +39,13 @@ protected:
   void set_parse_mode();
   void set_file_name();
 
+  void set_attachment_name();
+  void set_attachment_description();
+  void set_attachment_mime_type();
+  void add_attachment();
+  void delete_attachment();
+  void replace_attachment();
+
   void list_property_names();
   void list_property_names_for_table(const std::vector<property_element_c> &table, const std::string &title, const std::string &edit_spec);
 
